
// methods for CSWObject class (see CSWObject.h)

#include "CSWObject.h"

/**********************************************************************
public methods
**********************************************************************/

void CSWObject::init(LPDISPATCH pDispatch)
{

   my_lpdispatch = pDispatch;
   my_dispid_table.RemoveAll();

}

HRESULT CSWObject::get(LPOLESTR pszName, LPVARIANT pvRet)
{

   DISPID dispid;
   HRESULT hresult;
   DISPPARAMS NoArgs = {NULL, NULL, 0, 0};

   // invoke

   hresult = get_id(pszName, &dispid);
   if (hresult != S_OK) return hresult;
   VariantInit(pvRet);
   hresult = do_invoke(dispid,
                       DISPATCH_PROPERTYGET,
                       &NoArgs,
                       pvRet);
   return hresult;

}

HRESULT CSWObject::set(LPOLESTR pszName, LPVARIANT pvArg, LPVARIANT pvRet)
{

   DISPID dispid;
   HRESULT hresult;
   DISPPARAMS Args;

   // set up Args

   Args.cArgs = 1;
   Args.rgvarg = pvArg;
   Args.cNamedArgs = 0;
   Args.rgdispidNamedArgs = NULL;

   // invoke

   hresult = get_id(pszName, &dispid);
   if (hresult != S_OK) return hresult;
   VariantInit(pvRet);
   hresult = do_invoke(dispid,
                       DISPATCH_PROPERTYPUT,
                       &Args,
                       pvRet);
   return hresult;

}

HRESULT CSWObject::do_variant(LPOLESTR pszName, LPVARIANT pvArg, LPVARIANT pvRet)
{

   DISPID dispid;
   HRESULT hresult;
   DISPPARAMS Args;

   // set up Args

   Args.cArgs = 1;
   Args.rgvarg = pvArg;
   Args.cNamedArgs = 0;
   Args.rgdispidNamedArgs = NULL;

   // invoke

   hresult = get_id(pszName, &dispid);
   if (hresult != S_OK) return hresult;
   VariantInit(pvRet);
   hresult = do_invoke(dispid,
                       DISPATCH_METHOD,
                       &Args,
                       pvRet);
   return hresult;

}

HRESULT CSWObject::do_dispparams(LPOLESTR pszName, DISPPARAMS * pdArg, LPVARIANT pvRet)
{

   DISPID dispid;
   HRESULT hresult;

   // invoke

   hresult = get_id(pszName, &dispid);
   if (hresult != S_OK) return hresult;
   VariantInit(pvRet);
   hresult = do_invoke(dispid,
                       DISPATCH_METHOD,
                       pdArg,
                       pvRet);
   return hresult;

}

/**********************************************************************
private methods
**********************************************************************/

HRESULT CSWObject::get_id(LPOLESTR pszName, DISPID * dispid)
{

   DISPID d;
   HRESULT hresult;

   if (my_dispid_table.Lookup(pszName,d))
   {
      *dispid = d;
      return S_OK;
   }
   hresult = my_lpdispatch->GetIDsOfNames(IID_NULL,
                                          &pszName,
                                          1,
                                          LOCALE_USER_DEFAULT,
                                          &d);
   if (hresult != NOERROR) return hresult;
   my_dispid_table.SetAt(pszName,d);

   *dispid = d;
   return S_OK;

}

HRESULT CSWObject::do_invoke(DISPID dispid, WORD dispatch_type, DISPPARAMS * Args, LPVARIANT pvRet)
{

   EXCEPINFO excepinfo;
   HRESULT hresult;
   UINT argerr;

   hresult = my_lpdispatch->Invoke(dispid,
                                   IID_NULL,
                                   LOCALE_USER_DEFAULT,
                                   dispatch_type,
                                   Args,
                                   pvRet,
                                   &excepinfo,
                                   &argerr);

   return hresult;
}


